import json
from PyQt5 import Qt
from PyQt5.QtCore import pyqtSignal
from woocommerce.api import API
from settings import settings


class WooTableModel(Qt.QAbstractTableModel):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.headers = (
            (self.tr("#"), lambda x: x['number']),
            (self.tr("Status"), lambda x: x['status']),
            (self.tr("First name"), lambda x: x['shipping']['first_name']),
            (self.tr("Last name"), lambda x: x['shipping']['last_name']),
            (self.tr("City"), lambda x: x['shipping']['city']),
            (self.tr("Country"), lambda x: x['shipping']['country']),
            (self.tr("Phone"), lambda x: x['shipping'].get('phone', x['billing']['phone'])),
            (self.tr("E-mail"), lambda x: x['shipping'].get('email', x['billing']['email'])),
            (self.tr("Shipment number"), lambda x: x['shipping'].get('number', '')),
        )
        self._orders = []

    def rowCount(self, model_index=Qt.QModelIndex()):
        return len(self._orders)

    def columnCount(self, model_index=Qt.QModelIndex()):
        return len(self.headers)

    def headerData(self, section, orientation, role = Qt.Qt.DisplayRole):
        if role == Qt.Qt.DisplayRole:
            if orientation == Qt.Qt.Horizontal:
                return self.headers[section][0]
        elif role == Qt.Qt.CheckStateRole:
            return Qt.Qt.Checked
        return Qt.QVariant()

    def data(self, model_index:Qt.QModelIndex, role=Qt.Qt.DisplayRole):
        if role == Qt.Qt.DisplayRole:
            order = self._orders[model_index.row()]
            return self.headers[model_index.column()][1](order)
        elif role == Qt.Qt.UserRole:
            return self._orders[model_index.row()]
        return Qt.QVariant()

    def setData(self, index : Qt.QModelIndex, value : Qt.QVariant, role=Qt.Qt.EditRole):
        order = self._orders[index.row()]
        if role == Qt.Qt.UserRole:
            if index.column() == 1:
                wcapi = self.get_wcapi()
                try:
                    wcapi.post(
                        "orders/{}".format(order['id']),
                        {'status': value}
                    )
                    order['status'] = value
                except Exception as e:
                    Qt.QMessageBox.critical(
                        self.parent(),
                        self.tr("Make complete"),
                        str(e)
                    )
        return super().setData(index, value, role)

    def get_wcapi(self):
        wcapi = API(
            url=settings.get('woo_address', ''),
            consumer_key=settings.get('woo_consumer_key', ''),
            consumer_secret=settings.get('woo_consumer_secret', ''),
            wp_api=True,
            version="wc/v1"
        )
        return wcapi

    def update(self, checked=False):
        try:
            self.beginResetModel()
            wcapi = self.get_wcapi()
            #r = wcapi.get("orders?filter[status]=processing&per_page=500")
            r = wcapi.get("orders?per_page=100&status=processing")
            self._orders = r.json()
            self.endResetModel()
        except Exception as e:
            Qt.QMessageBox.critical(
                self.parent(),
                self.tr("Exception:"),
                str(e)
            )

    def txtFileDownloaded(self, path):
        if Qt.QMessageBox.question(
            self.parent(),
            self.tr("Import shipment numbers?"),
            self.tr("Import from: ") + path
        ) == Qt.QMessageBox.Yes:
            self.beginResetModel()
            with open(path, "r") as f:
                for index, line in enumerate(f.readlines()):
                    if index > 0:
                        columns = line.strip().split(';')
                        for order in self._orders:
                            if order['number'] == columns[2]:
                                try:
                                    print("Updating order: {}".format(order['number']))
                                    wcapi = self.get_wcapi()
                                    from gui.mainwindow import SHIPMENT_ACTION
                                    action = json.loads(settings.get('woo_shipment', SHIPMENT_ACTION))
                                    for k, v in action['context'].items():
                                        try:
                                            action['context'][k] = v.format(shipment_number=columns[0], **order)
                                        except:
                                            pass
                                    wcapi.post(action['endpoint'].format(
                                        shipment_number=columns[0],
                                        **order
                                    ), action['context'])
                                    order['shipping']['number'] = columns[0]
                                except Exception as e:
                                    Qt.QMessageBox.critical(
                                        self.parent(),
                                        self.tr("Exception:"),
                                        str(e)
                                    )
            self.endResetModel()

class WooTableView(Qt.QTableView):
    fillIn = pyqtSignal(object)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setModel(WooTableModel(self))
        self.setSelectionBehavior(Qt.QAbstractItemView.SelectRows)
        self.setEditTriggers(Qt.QAbstractItemView.NoEditTriggers)
        self.setContextMenuPolicy(Qt.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.handleCustomContextMenuRequested)
        self.fillInAction = Qt.QAction(self.tr("Fill in"), self)
        self.fillInAction.triggered.connect(self.handleFillInAction)

        self.completeAction = Qt.QAction(self.tr("Make complete"), self)
        self.completeAction.triggered.connect(self.handleCompleteAction)

        self.packageNumberAction = Qt.QAction(self.tr("Enter package number"), self)
        self.packageNumberAction.triggered.connect(self.handlePackageNumberAction)

    def handleFillInAction(self):
        index = self.selectedIndexes()[0]
        order = self.model().data(index, Qt.Qt.UserRole)
        self.fillIn.emit(order)

    def handleCompleteAction(self):
        for index in filter(lambda x: x.column() == 1, self.selectedIndexes()):
            self.model().setData(index, "completed", Qt.Qt.UserRole)

    def handlePackageNumberAction(self):
        for index in filter(lambda x: x.column() == 0, self.selectedIndexes()):
            pass

    def handleCustomContextMenuRequested(self, pos : Qt.QPoint):
        menu = Qt.QMenu(self)
        menu.addAction(self.fillInAction)
        menu.addSeparator()
        menu.addAction(self.completeAction)
        menu.popup(self.viewport().mapToGlobal(pos))
